@@ -663,10 +663,6 @@ fs_finalize (GObject *object)
 		g_hash_table_unref (priv->mtime_cache);
 	}
 
-	if (priv->iri_cache) {
-		g_hash_table_unref (priv->iri_cache);
-	}
-
 	G_OBJECT_CLASS (tracker_miner_fs_parent_class)->finalize (object);
 }
 
@@ -991,8 +987,6 @@ sparql_update_cb (GObject      *object,
 				 */
 				g_hash_table_insert (fs->private->iri_cache, g_object_ref (data->file), NULL);
 			}
-
-			g_object_unref (parent);
 		}
 	}
 
@@ -1125,48 +1119,22 @@ cache_query_cb (GObject	     *object,
 	}
 }
 
-static gboolean
-file_is_crawl_directory (TrackerMinerFS *fs,
-                         GFile          *file)
-{
-	GList *dirs;
-
-	/* Check whether file is a crawl directory itself */
-	dirs = fs->private->config_directories;
-
-	while (dirs) {
-		DirectoryData *data;
-
-		data = dirs->data;
-		dirs = dirs->next;
-
-		if (g_file_equal (data->file, file)) {
-			return TRUE;
-		}
-	}
-
-	return FALSE;
-}
-
 static void
 ensure_iri_cache (TrackerMinerFS *fs,
-                  GFile          *file)
+                  GFile          *parent)
 {
 	gchar *query, *uri;
 	CacheQueryData data;
-	GFile *parent;
 
 	g_hash_table_remove_all (fs->private->iri_cache);
 
-	parent = g_file_get_parent (file);
 	uri = g_file_get_uri (parent);
 
 	g_debug ("Generating IRI cache for folder: %s", uri);
 
-	query = g_strdup_printf ("SELECT ?url ?u { "
-	                         "  ?u nfo:belongsToContainer ?p ; "
-	                         "     nie:url ?url . "
-	                         "  ?p nie:url \"%s\" "
+	query = g_strdup_printf ("SELECT ?uri ?u { "
+	                         "  ?u nie:url ?uri . "
+	                         "  FILTER (tracker:uri-is-parent (\"%s\", ?uri)) "
 	                         "}",
 	                         uri);
 	g_free (uri);
@@ -1184,19 +1152,6 @@ ensure_iri_cache (TrackerMinerFS *fs,
 
 	g_main_loop_unref (data.main_loop);
 	g_hash_table_unref (data.values);
-
-	if (g_hash_table_size (data.values) == 0 &&
-	    file_is_crawl_directory (fs, file)) {
-		gchar *query_iri;
-
-		if (item_query_exists (fs, file, &query_iri, NULL)) {
-			g_hash_table_insert (data.values,
-			                     g_object_ref (file), query_iri);
-		}
-	}
-
-	g_object_unref (parent);
-	g_free (query);
 }
 
 static const gchar *
@@ -1222,7 +1177,6 @@ iri_cache_lookup (TrackerMinerFS *fs,
 		if (item_query_exists (fs, file, &query_iri, NULL)) {
 			g_hash_table_insert (fs->private->iri_cache,
 			                     g_object_ref (file), query_iri);
-			iri = query_iri;
 		} else {
 			g_hash_table_remove (fs->private->iri_cache, file);
 			iri = NULL;
@@ -1384,7 +1338,7 @@ item_add_or_update (TrackerMinerFS *fs,
 				fs->private->current_parent_urn = NULL;
 			}
 
-			ensure_iri_cache (fs, file);
+			ensure_iri_cache (fs, parent);
 		}
 
 		parent_urn = fs->private->current_parent_urn;
@@ -1634,9 +1588,9 @@ item_update_children_uri (TrackerMinerFS    *fs,
 
 	sparql = g_strdup_printf ("SELECT ?child ?url nie:mimeType(?child) WHERE { "
 				  "  ?child nie:url ?url . "
-                                  "  FILTER (tracker:uri-is-descendant (\"%s\", ?url)) "
+                                  "  FILTER (tracker:uri-is-descendant (\"%s\", ?uri)) "
 				  "}",
-				  source_uri);
+				  uri);
 
 	tracker_miner_execute_sparql (TRACKER_MINER (fs),
 	                              sparql,
@@ -1652,13 +1606,12 @@ item_move (TrackerMinerFS *fs,
            GFile          *file,
            GFile          *source_file)
 {
-	gchar     *uri, *source_uri;
+	gchar     *uri, *source_uri, *escaped_filename;
 	GFileInfo *file_info;
 	GString   *sparql;
 	RecursiveMoveData move_data;
 	ProcessData *data;
 	gchar *source_iri;
-	gchar *display_name;
 	gboolean source_exists;
 
 	iri_cache_invalidate (fs, file);
@@ -1747,7 +1700,7 @@ item_move (TrackerMinerFS *fs,
 	                        "} ",
 	                        source_iri, source_iri, source_iri);
 
-	display_name = tracker_sparql_escape (g_file_info_get_display_name (file_info));
+	escaped_filename = g_strescape (g_file_info_get_display_name (file_info), NULL);
 
 	g_string_append_printf (sparql,
 	                        "INSERT INTO <%s> {"
@@ -1756,10 +1709,10 @@ item_move (TrackerMinerFS *fs,
 	                        "       nie:isStoredAs <%s> "
 	                        "} ",
 	                        source_iri, source_iri,
-	                        display_name, uri,
+	                        escaped_filename, uri,
 	                        source_iri);
 
-	g_free (display_name);
+	g_free (escaped_filename);
 
 	move_data.main_loop = g_main_loop_new (NULL, FALSE);
 	move_data.sparql = sparql;
@@ -2202,7 +2155,7 @@ ensure_mtime_cache (TrackerMinerFS *fs,
 	CacheQueryData data;
 	GFile *parent;
 
-	if (G_UNLIKELY (!fs->private->mtime_cache)) {
+        if (G_UNLIKELY (!fs->private->mtime_cache)) {
 		fs->private->mtime_cache = g_hash_table_new_full (g_file_hash,
 		                                                  (GEqualFunc) g_file_equal,
 		                                                  (GDestroyNotify) g_object_unref,
@@ -2229,11 +2182,12 @@ ensure_mtime_cache (TrackerMinerFS *fs,
 
 	g_debug ("Generating mtime cache for folder: %s", uri);
 
-	query = g_strdup_printf ("SELECT ?url ?last { ?u nfo:belongsToContainer ?p ; "
-	                                                "nie:url ?url ; "
-	                                                "nfo:fileLastModified ?last . "
-	                                             "?p nie:url \"%s\" }", uri);
-
+	query = g_strdup_printf ("SELECT ?uri ?time { "
+	                         "  ?u nfo:fileLastModified ?time ; "
+	                         "     nie:url ?uri . "
+	                         "  FILTER (tracker:uri-is-parent (\"%s\", ?uri)) "
+	                         "}",
+	                         uri);
 	g_free (uri);
 
 	data.main_loop = g_main_loop_new (NULL, FALSE);
@@ -2244,35 +2198,9 @@ ensure_mtime_cache (TrackerMinerFS *fs,
 	                              NULL,
 	                              cache_query_cb,
 	                              &data);
-	g_free (query);
 
 	g_main_loop_run (data.main_loop);
 
-	if (g_hash_table_size (data.values) == 0 &&
-	    file_is_crawl_directory (fs, file)) {
-		/* File is a crawl directory itself, query its mtime directly */
-		uri = g_file_get_uri (file);
-
-		g_debug ("Folder %s is a crawl directory, generating mtime cache for it", uri);
-
-		query = g_strdup_printf ("SELECT ?url ?last "
-		                         "WHERE { "
-		                         "  ?u nfo:fileLastModified ?last ; "
-		                         "     nie:url ?url ; "
-		                         "     nie:url \"%s\" "
-		                         "}", uri);
-		g_free (uri);
-
-		tracker_miner_execute_sparql (TRACKER_MINER (fs),
-		                              query,
-		                              NULL,
-		                              cache_query_cb,
-		                              &data);
-		g_free (query);
-
-		g_main_loop_run (data.main_loop);
-	}
-
 	g_main_loop_unref (data.main_loop);
 	g_hash_table_unref (data.values);
 }
@@ -2770,7 +2698,7 @@ static gboolean
 crawl_directories_cb (gpointer user_data)
 {
 	TrackerMinerFS *fs = user_data;
-	gchar *path, *path_utf8;
+	gchar *path;
 	gchar *str;
 
 	if (fs->private->current_directory) {
@@ -2797,15 +2725,12 @@ crawl_directories_cb (gpointer user_data)
 	                                          fs->private->current_directory);
 
 	path = g_file_get_path (fs->private->current_directory->file);
-	path_utf8 = g_filename_to_utf8 (path, -1, NULL, NULL, NULL);
-	g_free (path);
 
 	if (fs->private->current_directory->recurse) {
-		str = g_strdup_printf ("Crawling recursively directory '%s'", path_utf8);
+		str = g_strdup_printf ("Crawling recursively directory '%s'", path);
 	} else {
-		str = g_strdup_printf ("Crawling single directory '%s'", path_utf8);
+		str = g_strdup_printf ("Crawling single directory '%s'", path);
 	}
-	g_free (path_utf8);
 
 	g_message ("%s", str);
 
@@ -2815,6 +2740,7 @@ crawl_directories_cb (gpointer user_data)
 	              "status", str,
 	              NULL);
 	g_free (str);
+	g_free (path);
 
 	if (tracker_crawler_start (fs->private->crawler,
 	                           fs->private->current_directory->file,
